/* -*- C++ -*-
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Interface to the WIND task class.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _WindTask_h
#define _WindTask_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include "SxStats.h"
#include "CkThread.h"
#include "vxworks/vxWorks.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_WIND
#include "sodecl.h"

class CkHistogram;
class WindNode;
class WindTask;
class WindMessage;

enum WindTaskStates {

    WindTaskDORMANT,
    WindTaskDELAYED,
    WindTaskPENDED,
    WindTaskSUSPENDED,
    WindTaskREADY,
    WindTaskRUNNING
};

// Indicates that the task pends on the send-side of a queue
#define WIND_PEND_SENDSIDE CK_THREAD_BIT0
// Indicates that the user-code entry point has been called
// (i.e. so we can longjmp() to it using restartContext).
#define WIND_TASK_STARTED  CK_THREAD_BIT1
// Indicates that the task is in a safe section
// (i.e. un-deletable until it exits from the section).
#define WIND_TASK_SAFE     CK_THREAD_BIT2
//
#define WIND_TASK_SAFE     CK_THREAD_BIT2

class WindTask : public CkThread {

protected:

    FUNCPTR startAddr;

    int startArgs[10];

    int options;

    int startPri;

    ITime lastStateChangeTime;

    CkHistogram *stateDistribs[5];

    WIND_TCB tcb,
	*pTcb;

    SxThreadContext restartContext;

    CkMutex safeMutex;

    CkCondVar safeCv;

    int safeLocks;

    void logTaskState(WindTaskStates s);

    virtual void dynamicExpose();

    virtual void dynamicConfigure(StProtoInfoMessage *mbuf);

    void kroot(run)();

    union _wakeUpArgs {

	struct {
	    WindMessage *msg;
	} queue;

    } wakeUpArgs;

    void *ptdSlots[32];

    static u_long ptdKeySlots;

public:

    static u_long seqGen;

    static const char *stateArray[6];

    static int allocPtdKey();

    static void freePtdKey(int key);

    WindTask(WIND_TCB *pTcb,
	     const char *name,
	     int priority,
	     FUNCPTR startAddr,
	     int options,
	     int startArgs[10],
	     WindNode *node,
	     int stackSize);

    virtual ~WindTask();

    WindNode *getNode() {
	return (WindNode *)CkThread::getNode();
    }

    WIND_TCB *getTcb() {
	return pTcb;
    }

    void setPtd(int key, void *data) {
	ptdSlots[key] = data;
    }

    void *getPtdPtr(int key) {
	return &ptdSlots[key];
    }

    void enterSafeSection();

    void exitSafeSection();

    void safeSync(WindTask *waiter);

    void setErrno(int errorNumber) {
	CkThread::setErrno(errorNumber);
	pTcb->errorStatus = errorNumber;
    }

    void msgSet(WindMessage *msg) {
	wakeUpArgs.queue.msg = msg;
    }

    WindMessage *msgGet() const {
	return wakeUpArgs.queue.msg;
    }

    void restart();

    virtual void body();

    virtual void setReady();

    virtual void setHalted();

    virtual void setDelayed();

    virtual void setPending();

    virtual void setRunning();

    virtual void setSuspended();

    virtual void setDormant();

    virtual void (*getStartAddr())(...);

    virtual void protoInit();

    virtual void startAsr(cksigmask_t sigs);
};

MakeGList(WindTask);

extern WindTaskGList allWindTasks;

#endif // !_WindTask_h
