/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 *
 * VxWorks is a registered trademark of Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Header replacement for semLib.h.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _vxworks_semLib_h
#define _vxworks_semLib_h

#include <ck.h>
#include "vxworks/windDefs.h"
#include "vxworks/errorLib.h"

#define SEM_Q_FIFO           0x0
#define SEM_Q_PRIORITY       0x1
#define SEM_DELETE_SAFE      0x4
#define SEM_INVERSION_SAFE   0x8
#define SEM_OPTION_MASK     (SEM_Q_FIFO|SEM_Q_PRIORITY| \
                             SEM_DELETE_SAFE|SEM_INVERSION_SAFE)
typedef enum {

	SEM_EMPTY =0,
	SEM_FULL =1

} SEM_B_STATE;

typedef ckhandle_t SEM_ID;

#ifdef __cplusplus
extern "C" {
#endif

SEM_ID semBCreate(int options,
		  SEM_B_STATE initialState);

SEM_ID semCCreate(int options,
		  int initialCount);

SEM_ID semMCreate(int options);

STATUS semDelete(SEM_ID semid);

STATUS semFlush(SEM_ID semid);

STATUS semGive(SEM_ID semid);

STATUS semTake(SEM_ID semid,
	       int ticks);

#ifdef __cplusplus
}
#endif

#endif	/* !_vxworks_semLib_h */
