/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 *
 * VxWorks is a registered trademark of Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Header replacement for taskLib.h.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _vxworks_taskLib_h
#define _vxworks_taskLib_h

#include <ck.h>
#include "vxworks/windDefs.h"
#include "vxworks/errorLib.h"

// The WIND task internal control block (which tends to be
// rather public in pre-6.0 versions of the VxWorks kernel).

typedef struct _WIND_TCB {

    const char *name;
    int options;
    int status;
    int priority;
    FUNCPTR entry;
    int errorStatus;

    ckhandle_t handle;	// Ck-specific

} WIND_TCB;

// Options (neither used nor enforced so far)

#define VX_UNBREAKABLE   0x2
#define VX_FP_TASK       0x8
#define VX_PRIVATE_ENV   0x80
#define VX_NO_STACK_FILL 0x100

#ifdef __cplusplus
extern "C" {
#endif

STATUS taskInit(WIND_TCB *pTcb,
		const char *name,
		int priority,
		int options,
		char *stackBase, // Ck: unused
		int stacksize,
		FUNCPTR entryPt,
		int arg1,
		int arg2,
		int arg3,
		int arg4,
		int arg5,
		int arg6,
		int arg7,
		int arg8,
		int arg9,
		int arg10);

STATUS taskActivate(int tid);

#define taskHookInit()

STATUS taskCreateHookAdd(FUNCPTR createHook);

STATUS taskSwitchHookAdd(FUNCPTR switchHook);

STATUS taskDeleteHookAdd(FUNCPTR deleteHook);

STATUS taskCreateHookDelete(FUNCPTR createHook);

STATUS taskSwitchHookDelete(FUNCPTR switchHook);

STATUS taskDeleteHookDelete(FUNCPTR deleteHook);

STATUS taskDelay (int ticks);

STATUS taskDelete(int tid);

STATUS taskDeleteForce(int tid);

void taskExit__(int status);
#if !defined(IN_CK) && defined(_CKCC_)
#define exit(status) taskExit__(status)
#endif

int taskIdDefault(int tid);

int taskIdSelf(void);

STATUS taskIdVerify(int tid);

STATUS taskLock(void);

STATUS taskPriorityGet(int tid,
		       int *pPriority);

STATUS taskPrioritySet(int tid,
		       int priority);

STATUS taskRestart(int tid);

STATUS taskResume(int tid);

STATUS taskSafe(void);

int taskSpawn(const char *name,
	      int priority,
	      int options,
	      int stacksize,
	      FUNCPTR entry,
	      int arg1,
	      int arg2,
	      int arg3,
	      int arg4,
	      int arg5,
	      int arg6,
	      int arg7,
	      int arg8,
	      int arg9,
	      int arg10);

STATUS taskSuspend(int tid);

WIND_TCB *taskTcb(int tid);

STATUS taskUnlock(void);

STATUS taskUnsafe(void);

#ifdef __cplusplus
}
#endif

#endif	/* !_vxworks_taskLib_h */
