/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Specialization of the CarbonKernel trace manager for the
 * WIND kernel's native API.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifdef __GNUG__
#pragma implementation
#endif // __GNUG__
#include <config.h>
#include <errno.h>
#include "CkManager.h"
#include "WindNode.h"
#include "WindProto.h"
#include "WindTrace.h"

static const char WindIntrFamily[] = "intLib";
static const char WindKernelFamily[] = "kernelLib";
static const char WindMsgQFamily[] = "msgQLib";
static const char WindSema4Family[] = "semLib";
static const char WindSysFamily[] = "sysLib";
static const char WindTaskFamily[] = "taskLib";
static const char WindTaskHookFamily[] = "taskHookLib";
static const char WindTaskInfoFamily[] = "taskInfoLib";
static const char WindTickFamily[] = "tickLib";
static const char WindWatchdogFamily[] = "wdLib";

CkTraceDef WindTraceInterface::traceDefTable[] = {

    { "intContext", WindIntrFamily }, // 0
    { "intCount", WindIntrFamily },
    { "intLevelSet", WindIntrFamily },
    { "intLock", WindIntrFamily },
    { "intUnlock", WindIntrFamily },
    { "kernelTimeSlice", WindKernelFamily },
    { "kernelVersion", WindKernelFamily },
    { "msgQCreate", WindMsgQFamily },
    { "msgQDelete", WindMsgQFamily },
    { "msgQNumMsgs", WindMsgQFamily },
    { "msgQReceive", WindMsgQFamily }, // 10
    { "msgQSend", WindMsgQFamily },
    { "semBCreate", WindSema4Family },
    { "semCCreate", WindSema4Family },
    { "semMCreate", WindSema4Family },
    { "semDelete", WindSema4Family },
    { "semFlush", WindSema4Family },
    { "semGive", WindSema4Family },
    { "semTake", WindSema4Family },
    { "sysClkDisable", WindSysFamily },
    { "sysClkEnable", WindSysFamily },	// 20
    { "sysClkRateGet", WindSysFamily },
    { "taskActivate", WindTaskFamily },
    { "taskDelay", WindTaskFamily },
    { "taskDelete", WindTaskFamily },
    { "taskDeleteForce", WindTaskFamily },
    { "taskIdSelf", WindTaskFamily },
    { "taskIdVerify", WindTaskFamily },
    { "taskInit", WindTaskFamily },
    { "taskLock", WindTaskFamily },
    { "taskPriorityGet", WindTaskFamily }, // 30
    { "taskPrioritySet", WindTaskFamily },
    { "taskRestart", WindTaskFamily },
    { "taskResume", WindTaskFamily },
    { "taskSafe", WindTaskFamily },
    { "taskSpawn", WindTaskFamily },
    { "taskSuspend", WindTaskFamily },
    { "taskTcb", WindTaskFamily },
    { "taskUnlock", WindTaskFamily },
    { "taskUnsafe", WindTaskFamily },
    { "exit", WindTaskFamily }, // 40
    { "taskCreateHookAdd", WindTaskHookFamily },
    { "taskCreateHookDelete", WindTaskHookFamily },
    { "taskDeleteHookAdd", WindTaskHookFamily },
    { "taskDeleteHookDelete", WindTaskHookFamily },
    { "taskSwitchHookAdd", WindTaskHookFamily },
    { "taskSwitchHookDelete", WindTaskHookFamily },
    { "taskIdDefault", WindTaskInfoFamily },
    { "taskIsReady", WindTaskInfoFamily },
    { "taskIsSuspended", WindTaskInfoFamily },
    { "taskName", WindTaskInfoFamily }, // 50
    { "taskNameToId", WindTaskInfoFamily },
    { "tickGet", WindTickFamily },
    { "tickSet", WindTickFamily },
    { "wdCancel", WindWatchdogFamily },
    { "wdCreate", WindWatchdogFamily },
    { "wdDelete", WindWatchdogFamily },
    { "wdStart", WindWatchdogFamily }
};

int WindTraceInterface::traceDefCount =
sizeof(WindTraceInterface::traceDefTable) /
sizeof(CkTraceDef);

CkErrorDef WindTraceInterface::errorDefTable[] = {

    { "OK", 0 },
    { "S_objLib_OBJ_ID_ERROR", S_objLib_OBJ_ID_ERROR },
    { "S_objLib_OBJ_DELETED", S_objLib_OBJ_DELETED },
    { "S_objLib_OBJ_UNAVAILABLE", S_objLib_OBJ_UNAVAILABLE },
    { "S_objLib_OBJ_TIMEOUT", S_objLib_OBJ_TIMEOUT },
    { "S_taskLib_NAME_NOT_FOUND", S_taskLib_NAME_NOT_FOUND },
    { "S_taskLib_TASK_HOOK_NOT_FOUND", S_taskLib_TASK_HOOK_NOT_FOUND },
    { "S_taskLib_ILLEGAL_PRIORITY", S_taskLib_ILLEGAL_PRIORITY },
    { "S_semLib_INVALID_STATE", S_semLib_INVALID_STATE },
    { "S_semLib_INVALID_OPTION", S_semLib_INVALID_OPTION },
    { "S_semLib_INVALID_QUEUE_TYPE", S_semLib_INVALID_QUEUE_TYPE },
    { "S_semLib_INVALID_OPERATION", S_semLib_INVALID_OPERATION },
    { "S_msgQLib_INVALID_MSG_LENGTH", S_msgQLib_INVALID_MSG_LENGTH },
    { "S_msgQLib_NON_ZERO_TIMEOUT_AT_INT_LEVEL", S_msgQLib_NON_ZERO_TIMEOUT_AT_INT_LEVEL },
    { "S_msgQLib_INVALID_QUEUE_TYPE", S_msgQLib_INVALID_QUEUE_TYPE },
    { "S_intLib_NOT_ISR_CALLABLE", S_intLib_NOT_ISR_CALLABLE },
    { "EINTR", EINTR }
};

int WindTraceInterface::errorDefCount =
sizeof(WindTraceInterface::errorDefTable) /
sizeof(CkErrorDef);

WindTraceInterface::WindTraceInterface (WindNode *_node) :
    CkTraceInterface(_node->getOS(), // Use o/s' canonical name
		     traceDefTable,
		     traceDefCount,
		     errorDefTable,
		     errorDefCount)
{}
