#  This file is part of the VxWorks simulation model for CarbonKernel.
#  Copyright (C) 2001  Philippe Gerum.
# 
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  Author(s): rpm
# 
#  Contributor(s):

proc WindMutex:attach {context name private} {

    global $context.info
    set $context.info(present) yes
    return [list [list {WIND kernel} Mutexes $name] mutex]
}

proc WindMutex:detach {context} {

    global $context.info
    unset $context.info
    catch { destroy $context }
}

proc WindMutex:show {context nid name} {

    if {[winfo exists $context]} {
	wm deiconify $context
	raise $context
    }

    WindMutex:pollInfo $context
}

proc WindMutex:event {context event} {

    switch $event {
	SimulationHeldEvent {
	    if {[catch { wm state $context } state] == 0} {
		if {$state != "withdrawn"} {
		    WindMutex:pollInfo $context
		}
	    }
	}
    }
}

proc WindMutex:update {context nid name info} {

    global $context.info

    # dispatch the information block contents
    # to the per-context data array...
    foreach e $info {
	set item [lindex $e 0]
	set value [lindex $e 1]
	set $context.info($item) $value
    }

    set $context.info(current) $info

    if {[winfo exists $context]} {
	set threadfrm [$context.sw2 subwidget frame]
	WindTask:fillWaitList $context $threadfrm.list
	return
    }

    # if the mutex object has never been displayed before,
    # build a brand new toplevel for it...

    toplevel $context
    wm title $context "WIND mutex: $name"
    cascadeWindow $context
    wm protocol $context WM_DELETE_WINDOW \
	"wm withdraw $context"

    # State

    tixLabelFrame $context.sw1 -labelside none -options {
	label.padx 5
    }

    pack $context.sw1 -expand no -fill both
    set statfrm [$context.sw1 subwidget frame]
    tixOptionMenu $statfrm.state -label State
    $statfrm.state add command FREE -label FREE
    $statfrm.state add command LOCKED -label LOCKED
    pack $statfrm.state -anchor w -pady 5 -padx 5
    $statfrm.state config -variable $context.info(state)
    $statfrm.state subwidget label config -width 6

    # Pending threads

    tixLabelFrame $context.sw2 -label "Pending list" \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw2 -expand true -fill both
    set threadfrm [$context.sw2 subwidget frame]
    tixScrolledListBox $threadfrm.list -scrollbar auto \
	-command "WindTask:pickWaitList $context $threadfrm.list"
    pack $threadfrm.list -expand true -fill both
    $threadfrm.list subwidget listbox config -height 5 -bd 2
    WindTask:fillWaitList $context $threadfrm.list

    # Button box

    tixButtonBox $context.bbox -orientation horizontal -relief flat -bd 0
    $context.bbox add update -text Update \
	-command "WindMutex:pollInfo $context"
    $context.bbox add apply -text Apply \
	-command "WindMutex:applyChanges $context"
    $context.bbox add cancel -text Close \
	-command "wm withdraw $context"
    pack $context.bbox -side bottom -fill x
}

proc WindMutex:pollInfo {context} {
    TkRequest $context GetObjectInfo all
}

proc WindMutex:applyChanges {context} {

    global $context.info

    set oldinfo [set $context.info(current)]
    set newval [list state [set $context.info(state)]]

    if {[lsearch -exact $oldinfo $newval] == -1} {
	TkRequest $context ConfigureObject $newval
    }
}
