/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Testing VxWorks services:
 * - kernelTimeSlice
 * - taskSpawn
 * - tickGet
 * - tickSet
 *
 * WARNING: This test depends on the default WIND node warp factor (2.0)
 * and RTC frequency (100 Hz) as defined by the test configuration file
 * (TestConfig).
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <vxworks/vxWorks.h>

static ckhandle_t rootHandle,
    sliceHandles[3];

void sliceTask (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());

    sliceHandles[a0] = pTcb->handle;	/* WIND_TCB -> Ck handle */

    for (;;)
	{
	int n;

	TEST_MARK();

	for (n = 0; n < 10000; n++)
	    ;
	}
}

void rootTask (int a0, int a1, int a2, int a3, int a4,
	       int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb;
    int rc;

    TEST_START(0);

    pTcb = taskTcb(taskIdSelf());
    TEST_ASSERT(pTcb != NULL);
    rootHandle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    kernelTimeSlice(2);

    /* The root task starts with a priority of 0 */

    taskSpawn("Slicer1",
	      20,
	      0,
	      32768,
	      (FUNCPTR)sliceTask,
	      0,0,0,0,0,0,0,0,0,0);

    taskSpawn("Slicer2",
	      20,
	      0,
	      32768,
	      (FUNCPTR)sliceTask,
	      1,0,0,0,0,0,0,0,0,0);

    taskSpawn("Slicer3",
	      20,
	      0,
	      32768,
	      (FUNCPTR)sliceTask,
	      2,0,0,0,0,0,0,0,0,0);

    taskDelay(6);

    tickSet(50);

    TEST_ASSERT(tickGet() == 50);

    TEST_MARK();

    kernelTimeSlice(0);

    TEST_MARK();

    TEST_ASSERT_OK(taskDelay(6));

    TEST_MARK();

    TEST_CHECK_SEQUENCE(SEQ(sliceHandles[0],15),
			SEQ(sliceHandles[1],15),
			SEQ(sliceHandles[2],15),
			SEQ(rootHandle,2),
			SEQ(sliceHandles[0],45),
			SEQ(rootHandle,1),
			END_SEQ);

    TEST_FINISH();
}
