/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Testing VxWorks services:
 * - semCCreate
 * - semDelete
 * - semGive
 * - semTake
 * - taskLock
 * - taskUnlock
 * - taskNameToId
 * - taskSuspend
 * - taskResume
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <vxworks/vxWorks.h>

static ckhandle_t task1handle, task2handle;

static SEM_ID semid;

void sem1Task  (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());

    TEST_ASSERT(pTcb != NULL);

    task1handle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    semid = semCCreate(0xffffffff,0);
    TEST_ASSERT(semid == 0 && errno == S_semLib_INVALID_OPTION);

    semid = semCCreate(SEM_Q_FIFO,0);
    TEST_ASSERT(semid != 0);

    TEST_MARK();

    TEST_ASSERT_OK(semTake(semid,WAIT_FOREVER)); /* wait for semGive() */
    
    TEST_MARK();

    TEST_ASSERT_OK(semGive(semid));

    TEST_MARK();

    TEST_ASSERT_OK(taskSuspend(0));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(semid,10));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(semid,NO_WAIT));

    TEST_MARK();

    TEST_ASSERT(semTake(semid,10) == ERROR && errno == S_objLib_OBJ_TIMEOUT);

    TEST_CHECK_SEQUENCE(SEQ(task1handle,1),
			SEQ(task2handle,1),
			SEQ(task1handle,2),
			SEQ(task2handle,3),
			SEQ(task1handle,3),
			END_SEQ);

    TEST_ASSERT_OK(semDelete(semid));

    TEST_ASSERT(!ckObjectExists(semid));

    TEST_FINISH();
}

void sem2Task  (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());

    TEST_ASSERT(pTcb != NULL);

    task2handle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_MARK();

    TEST_ASSERT_OK(semGive(semid));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(semid,WAIT_FOREVER)); /* wait for semGive() */

    TEST_ASSERT_OK(taskLock());

    TEST_ASSERT_OK(taskResume(taskNameToId("Test1")));

    TEST_ASSERT_OK(semGive(semid));

    TEST_MARK();

    TEST_ASSERT_OK(semGive(semid));

    TEST_MARK();

    TEST_ASSERT_OK(taskUnlock());

    TEST_ASSERT_OK(taskSuspend(0));

    TEST_MARK();		/* Should not pass this mark */
}

void rootTask (int a0, int a1, int a2, int a3, int a4,
	       int a5, int a6, int a7, int a8, int a9)
{
    TEST_START(0);

    taskSpawn("Test1",
	      19,
	      0,
	      32768,
	      (FUNCPTR)sem1Task,
	      0,0,0,0,0,0,0,0,0,0);

    taskSpawn("Test2",
	      20,
	      0,
	      32768,
	      (FUNCPTR)sem2Task,
	      0,0,0,0,0,0,0,0,0,0);

    TEST_ASSERT_OK(taskSuspend(0));

    TEST_MARK(); /* Should not pass this mark */
}
