/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Testing VxWorks services:
 * - wdCreate
 * - wdDelete
 * - wdCancel
 * - wdStart
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <vxworks/vxWorks.h>

static ckhandle_t rootHandle,
    wdogHandle;

static WDOG_ID wid;

void watchdogHandler (int arg)

{
    wdogHandle = TEST_GET_ID();
    TEST_ASSERT(arg == 0x25262728);
    TEST_MARK();
}

void rootTask (int a0, int a1, int a2, int a3, int a4,
	       int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb;

    TEST_START(0);

    pTcb = taskTcb(taskIdSelf());
    TEST_ASSERT(pTcb != NULL);
    rootHandle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_MARK();

    wid = wdCreate();

    TEST_ASSERT(wid != 0);

    TEST_ASSERT(wdStart(0,10,(FUNCPTR)watchdogHandler,0) == ERROR &&
		errno == S_objLib_OBJ_ID_ERROR);

    TEST_MARK();

    TEST_ASSERT_OK(wdStart(wid,10,(FUNCPTR)watchdogHandler,0x25262728));

    TEST_MARK();

    TEST_ASSERT_OK(taskDelay(20));

    TEST_MARK();

    TEST_ASSERT(wdDelete(0) == ERROR && errno == S_objLib_OBJ_ID_ERROR);

    TEST_MARK();

    TEST_ASSERT_OK(wdDelete(wid));

    TEST_MARK();

    TEST_ASSERT(!ckObjectExists(wid));

    TEST_CHECK_SEQUENCE(SEQ(rootHandle,3),
			SEQ(wdogHandle,1),
			SEQ(rootHandle,3),
			END_SEQ);

    TEST_FINISH();
}
