/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Testing VxWorks services:
 * - taskActivate
 * - taskDelete
 * - taskDeleteForce
 * - taskIdVerify
 * - taskInit
 * - taskPriorityGet
 * - taskPrioritySet
 * - taskTcb
 * - taskSafe
 * - taskUnsafe
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <vxworks/vxWorks.h>

static ckhandle_t rootHandle, peerHandle;

static WIND_TCB peerTcb;

void peerTask  (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());
    TEST_ASSERT(pTcb == &peerTcb);
    peerHandle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_MARK();

    TEST_ASSERT_OK(taskSafe());

    TEST_MARK();

    TEST_ASSERT_OK(taskPrioritySet(taskIdSelf(),21));

    TEST_MARK();

    TEST_ASSERT_OK(taskUnsafe());

    TEST_MARK();
}

void rootTask (int a0, int a1, int a2, int a3, int a4,
	       int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb;
    int prio = 0;

    TEST_START(0);

    pTcb = taskTcb(taskIdSelf());
    TEST_ASSERT(pTcb != NULL);
    rootHandle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_ASSERT(taskInit(&peerTcb,
			 "peerTask",
			 -1,
			 0,
			 NULL,
			 32768,
			 (FUNCPTR)peerTask,
			 0,0,0,0,0,0,0,0,0,0) == ERROR &&
		errno == S_taskLib_ILLEGAL_PRIORITY);

    TEST_ASSERT_OK(taskInit(&peerTcb,
			    "peerTask",
			    19,
			    0,
			    NULL,
			    32768,
			    (FUNCPTR)peerTask,
			    0,0,0,0,0,0,0,0,0,0));

    TEST_ASSERT_OK(taskPrioritySet(taskIdSelf(),20));

    TEST_MARK();

    TEST_ASSERT(taskPriorityGet(taskIdSelf(),&prio) == OK && prio == 20);

    TEST_MARK();

    TEST_ASSERT(taskIdVerify(0) == ERROR && errno == S_objLib_OBJ_ID_ERROR);

    TEST_ASSERT_OK(taskIdVerify((int)&peerTcb));

    TEST_ASSERT_OK(taskActivate((int)&peerTcb));

    TEST_MARK();

    TEST_CHECK_SEQUENCE(SEQ(rootHandle,2),
			SEQ(peerHandle,2),
			SEQ(rootHandle,1),
			END_SEQ);

    TEST_ASSERT_OK(taskDelete((int)&peerTcb));

    TEST_ASSERT(!ckObjectExists(peerHandle));

    TEST_FINISH();
}
