/*
 * This file is part of the VxWorks simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Testing VxWorks services:
 * - semMCreate
 * - semBCreate
 * - semGive
 * - semTake
 * - semFlush
 * - taskIsReady
 * - taskIsSuspended
 * - taskNameToId
 *
 * WARNING: This test depends on the default WIND node warp factor (2.0)
 * and RTC frequency (100 Hz) as defined by the test configuration file
 * (TestConfig).
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <vxworks/vxWorks.h>

static ckhandle_t rootHandle, task1Handle, task2Handle;

static SEM_ID bsemid, mutexid;

void sem1Task  (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());
    int n;

    TEST_ASSERT(pTcb != NULL);
    task1Handle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_ASSERT(semFlush(mutexid) == ERROR &&
		errno == S_semLib_INVALID_OPERATION);

    TEST_ASSERT_OK(semFlush(bsemid));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(mutexid,WAIT_FOREVER));

    TEST_MARK();

    TEST_ASSERT_OK(semGive(bsemid));

    TEST_MARK();

    TEST_ASSERT_OK(semGive(mutexid));

    TEST_MARK(); /* Should not pass this mark */
}

void sem2Task  (int a0, int a1, int a2, int a3, int a4,
		int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb = taskTcb(taskIdSelf());
    int n;

    TEST_ASSERT(pTcb != NULL);
    task2Handle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    TEST_ASSERT(taskNameToId("Test2") == (int)pTcb);

    TEST_MARK();

    TEST_ASSERT_OK(semTake(bsemid,WAIT_FOREVER));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(bsemid,WAIT_FOREVER));

    TEST_MARK();

    TEST_ASSERT_OK(taskResume(a0));

    for (n = 0; n < 1000000; n++)
	TEST_MARK();

    TEST_MARK(); /* Should not pass this mark */
}

void rootTask (int a0, int a1, int a2, int a3, int a4,
	       int a5, int a6, int a7, int a8, int a9)
{
    WIND_TCB *pTcb;

    TEST_START(0);

    pTcb = taskTcb(taskIdSelf());
    TEST_ASSERT(pTcb != NULL);
    rootHandle = pTcb->handle;	/* WIND_TCB -> Ck handle */

    bsemid = semBCreate(0xffffffff,0);
    TEST_ASSERT(bsemid == 0 && errno == S_semLib_INVALID_OPTION);

    bsemid = semBCreate(SEM_Q_PRIORITY,0);
    TEST_ASSERT(bsemid != 0);

    mutexid = semMCreate(0xffffffff);
    TEST_ASSERT(mutexid == 0 && errno == S_semLib_INVALID_OPTION);

    mutexid = semMCreate(SEM_Q_PRIORITY|SEM_DELETE_SAFE|SEM_INVERSION_SAFE);
    TEST_ASSERT(mutexid != 0);

    taskSpawn("Test1",
	      20,		/* low-pri */
	      0,
	      32768,
	      (FUNCPTR)sem1Task,
	      taskIdSelf(),0,0,0,0,0,0,0,0,0);

    taskSpawn("Test2",
	      10,		/* intermediate-pri */
	      0,
	      32768,
	      (FUNCPTR)sem2Task,
	      taskIdSelf(),0,0,0,0,0,0,0,0,0);

    /* This runs at high-pri */

    TEST_MARK();

    TEST_ASSERT_OK(semTake(bsemid,WAIT_FOREVER));

    TEST_MARK();

    TEST_ASSERT_OK(taskSuspend(0));

    TEST_MARK();

    TEST_ASSERT_OK(semTake(mutexid,WAIT_FOREVER));

    TEST_MARK();

    TEST_ASSERT_OK(semGive(mutexid));

    TEST_MARK();

    TEST_ASSERT(taskIsReady(taskNameToId("Test2")));

    TEST_ASSERT(!taskIsSuspended(taskNameToId("Test2")));

    TEST_ASSERT_OK(taskDelay(1));

    TEST_MARK();

    TEST_CHECK_SEQUENCE(SEQ(rootHandle,1),
			SEQ(task2Handle,1),
			SEQ(rootHandle,1),
			SEQ(task2Handle,1),
			SEQ(task1Handle,2),
			SEQ(task2Handle,1),
			SEQ(rootHandle,1),
			SEQ(task1Handle,1),
			SEQ(rootHandle,2),
			SEQ(task2Handle,73823),
			SEQ(rootHandle,1),
			END_SEQ);
    TEST_FINISH();
}
